@@ -40,6 +40,9 @@ struct flb_net_setup {
     /* max time in seconds to wait for a established connection */
     int connect_timeout;
 
+    /* connect timeout log error (default: true) */
+    int connect_timeout_log_error;
+
     /* network interface to bind and use to send data */
     flb_sds_t source_address;
 
@@ -49,8 +52,11 @@ struct flb_net_setup {
     /* dns mode : TCP or UDP */
     char *dns_mode;
 
-    /* dns reolver : LEGACY or ASYNC */
+    /* dns resolver : LEGACY or ASYNC */
     char *dns_resolver;
+
+    /* prioritize ipv4 results when trying to establish a connection*/
+    int   dns_prefer_ipv4;
 };
 
 /* Defines a host service and it properties */
