@@ -475,11 +475,13 @@ static int cb_aws_filter(const void *data, size_t bytes,
                          const char *tag, int tag_len,
                          void **out_buf, size_t *out_size,
                          struct flb_filter_instance *f_ins,
+                         struct flb_input_instance *i_ins,
                          void *context,
                          struct flb_config *config)
 {
     struct flb_filter_aws *ctx = context;
     (void) f_ins;
+    (void) i_ins;
     (void) config;
     size_t off = 0;
     int i = 0;
