@@ -390,6 +390,7 @@ static int cb_expect_filter(const void *data, size_t bytes,
                             const char *tag, int tag_len,
                             void **out_buf, size_t *out_bytes,
                             struct flb_filter_instance *f_ins,
+                            struct flb_input_instance *i_ins,
                             void *filter_context,
                             struct flb_config *config)
 {
@@ -399,6 +400,7 @@ static int cb_expect_filter(const void *data, size_t bytes,
     (void) out_buf;
     (void) out_bytes;
     (void) f_ins;
+    (void) i_ins;
     (void) config;
     msgpack_object map;
     msgpack_object root;
