@@ -1283,11 +1283,13 @@ static int cb_modify_filter(const void *data, size_t bytes,
                             const char *tag, int tag_len,
                             void **out_buf, size_t * out_size,
                             struct flb_filter_instance *f_ins,
+                            struct flb_input_instance *i_ins,
                             void *context, struct flb_config *config)
 {
     msgpack_unpacked result;
     size_t off = 0;
     (void) f_ins;
+    (void) i_ins;
     (void) config;
 
     struct filter_modify_ctx *ctx = context;
