@@ -254,6 +254,7 @@ static int cb_modifier_filter(const void *data, size_t bytes,
                               const char *tag, int tag_len,
                               void **out_buf, size_t *out_size,
                               struct flb_filter_instance *f_ins,
+                              struct flb_input_instance *i_ins,
                               void *context,
                               struct flb_config *config)
 {
@@ -265,6 +266,7 @@ static int cb_modifier_filter(const void *data, size_t bytes,
     int map_num          = 0;
     bool_map_t *bool_map = NULL;
     (void) f_ins;
+    (void) i_ins;
     (void) config;
     struct flb_time tm;
     struct modifier_record *mod_rec;
