@@ -325,6 +325,9 @@ static int process_record(const char *tag, int tag_len, msgpack_object map,
 
     mk_list_foreach(head, &ctx->rules) {
         rule = mk_list_entry(head, struct rewrite_rule, _head);
+        if (rule) {
+            *keep = rule->keep_record;
+        }
         ret = flb_ra_regex_match(rule->ra_key, map, rule->regex, &result);
         if (ret < 0) { /* no match */
             rule = NULL;
@@ -361,7 +364,7 @@ static int process_record(const char *tag, int tag_len, msgpack_object map,
         return FLB_FALSE;
     }
 
-    *keep = rule->keep_record;
+
     return FLB_TRUE;
 }
 
@@ -369,6 +372,7 @@ static int cb_rewrite_tag_filter(const void *data, size_t bytes,
                                  const char *tag, int tag_len,
                                  void **out_buf, size_t *out_bytes,
                                  struct flb_filter_instance *f_ins,
+                                 struct flb_input_instance *i_ins,
                                  void *filter_context,
                                  struct flb_config *config)
 {
@@ -388,6 +392,7 @@ static int cb_rewrite_tag_filter(const void *data, size_t bytes,
     msgpack_unpacked result;
     struct flb_rewrite_tag *ctx = (struct flb_rewrite_tag *) filter_context;
     (void) config;
+    (void) i_ins;
 
 #ifdef FLB_HAVE_METRICS
     ts = cmt_time_now();
