@@ -41,6 +41,7 @@ static int cb_stdout_filter(const void *data, size_t bytes,
                             const char *tag, int tag_len,
                             void **out_buf, size_t *out_bytes,
                             struct flb_filter_instance *f_ins,
+                            struct flb_input_instance *i_ins,
                             void *filter_context,
                             struct flb_config *config)
 {
@@ -49,6 +50,7 @@ static int cb_stdout_filter(const void *data, size_t bytes,
     (void) out_buf;
     (void) out_bytes;
     (void) f_ins;
+    (void) i_ins;
     (void) filter_context;
     (void) config;
     struct flb_time tmp;
