@@ -207,6 +207,11 @@ static int in_fw_exit(void *data, struct flb_config *config)
 
 /* Configuration properties map */
 static struct flb_config_map config_map[] = {
+   {
+    FLB_CONFIG_MAP_STR, "tag_prefix", NULL,
+    0, FLB_TRUE, offsetof(struct flb_in_fw_config, tag_prefix),
+    "Prefix incoming tag with the defined value."
+   },
    {
     FLB_CONFIG_MAP_STR, "unix_path", NULL,
     0, FLB_TRUE, offsetof(struct flb_in_fw_config, unix_path),
