@@ -264,7 +264,7 @@ static int cb_cloudwatch_init(struct flb_output_instance *ins,
         ctx->aws_provider = flb_sts_provider_create(config,
                                                     ctx->sts_tls,
                                                     ctx->base_aws_provider,
-                                                    NULL,
+                                                    (char *) ctx->external_id,
                                                     (char *) ctx->role_arn,
                                                     session_name,
                                                     (char *) ctx->region,
@@ -590,6 +590,13 @@ static struct flb_config_map config_map[] = {
      "Specify a custom endpoint for the STS API, can be used with the role_arn parameter"
     },
 
+    {
+     FLB_CONFIG_MAP_STR, "external_id", NULL,
+     0, FLB_TRUE, offsetof(struct flb_cloudwatch, external_id),
+    "Specify an external ID for the STS API, can be used with the role_arn parameter if your role "
+     "requires an external ID."
+    },
+
     {
      FLB_CONFIG_MAP_STR, "metric_namespace", NULL,
      0, FLB_FALSE, 0,
