@@ -232,7 +232,7 @@ static inline int handle_output_event(flb_pipefd_t fd, uint64_t ts,
         flb_task_retry_clean(task, ins);
         flb_task_users_dec(task, FLB_TRUE);
     }
-    else if (ret == FLB_RETRY) {
+    else if (ret == FLB_RETRY && config->is_running && !config->is_shutting_down) {
         if (ins->retry_limit == FLB_OUT_RETRY_NONE) {
             /* cmetrics: output_dropped_records_total */
             cmt_counter_add(ins->cmt_dropped_records, ts, task->records,
@@ -695,6 +695,11 @@ int flb_engine_start(struct flb_config *config)
             if (event->type == FLB_ENGINE_EV_CORE) {
                 ret = flb_engine_handle_event(event->fd, event->mask, config);
                 if (ret == FLB_ENGINE_STOP) {
+                    if (config->grace_count == 0) {
+                        flb_warn("[engine] service will shutdown in max %u seconds",
+                                 config->grace);
+                    }
+
                     /*
                      * We are preparing to shutdown, we give a graceful time
                      * of 'config->grace' seconds to process any pending event.
@@ -702,19 +707,30 @@ int flb_engine_start(struct flb_config *config)
                     event = &config->event_shutdown;
                     event->mask = MK_EVENT_EMPTY;
                     event->status = MK_EVENT_NONE;
+
+                    /*
+                     * Configure a timer of 1 second, on expiration the code will
+                     * jump into the FLB_ENGINE_SHUTDOWN condition where it will
+                     * check if the grace period has finished, or if there are
+                     * any remaining tasks.
+                     *
+                     * If no tasks exists, there is no need to wait for the maximum
+                     * grace period.
+                     */
                     config->shutdown_fd = mk_event_timeout_create(evl,
-                                                                  config->grace,
+                                                                  1,
                                                                   0,
                                                                   event);
-                    flb_warn("[engine] service will stop in %u seconds", config->grace);
                 }
                 else if (ret == FLB_ENGINE_SHUTDOWN) {
-                    flb_info("[engine] service stopped");
                     if (config->shutdown_fd > 0) {
                         mk_event_timeout_destroy(config->evl,
                                                  &config->event_shutdown);
                     }
 
+                    /* Increase the grace counter */
+                    config->grace_count++;
+
                     /*
                      * Grace period has finished, but we need to check if there is
                      * any pending running task. A running task is associated to an
@@ -723,13 +739,18 @@ int flb_engine_start(struct flb_config *config)
                      * wait again for the grace period and re-check again.
                      */
                     ret = flb_task_running_count(config);
-                    if (ret > 0) {
-                        flb_warn("[engine] shutdown delayed, grace period has "
-                                 "finished but some tasks are still running.");
-                        flb_task_running_print(config);
+                    if (ret > 0 && config->grace_count < config->grace) {
+                        if (config->grace_count == 1) {
+                            flb_task_running_print(config);
+                        }
                         flb_engine_exit(config);
                     }
                     else {
+                        if (ret > 0) {
+                            flb_task_running_print(config);
+                        }
+                        flb_info("[engine] service has stopped (%i pending tasks)",
+                                 ret);
                         ret = config->exit_status_code;
                         flb_engine_shutdown(config);
                         config = NULL;
@@ -836,6 +857,7 @@ int flb_engine_exit(struct flb_config *config)
     uint64_t val = FLB_ENGINE_EV_STOP;
 
     config->is_ingestion_active = FLB_FALSE;
+    config->is_shutting_down = FLB_TRUE;
 
     flb_input_pause_all(config);
 
