@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -32,7 +32,8 @@ int flb_socket_error(int fd)
 
     ret = getsockopt(fd, SOL_SOCKET, SO_ERROR, &error, &slen);
     if (ret == -1) {
-        flb_error("[socket] could not validate socket status for #i", fd);
+        flb_debug("[socket] could not validate socket status for #%i (don't worry)",
+                  fd);
         return -1;
     }
 
