@@ -46,6 +46,12 @@ struct flb_config_map upstream_net[] = {
      "Select the primary DNS resolver type (LEGACY or ASYNC)"
     },
 
+    {
+     FLB_CONFIG_MAP_BOOL, "net.dns.prefer_ipv4", "false",
+     0, FLB_TRUE, offsetof(struct flb_net_setup, dns_prefer_ipv4),
+     "Prioritize IPv4 DNS results when trying to establish a connection"
+    },
+
     {
      FLB_CONFIG_MAP_BOOL, "net.keepalive", "true",
      0, FLB_TRUE, offsetof(struct flb_net_setup, keepalive),
@@ -65,6 +71,13 @@ struct flb_config_map upstream_net[] = {
      "includes the TLS handshake"
     },
 
+    {
+     FLB_CONFIG_MAP_BOOL, "net.connect_timeout_log_error", "true",
+     0, FLB_TRUE, offsetof(struct flb_net_setup, connect_timeout_log_error),
+     "On connection timeout, specify if it should log an error. When disabled, "
+     "the timeout is logged as a debug message"
+    },
+
     {
      FLB_CONFIG_MAP_STR, "net.source_address", NULL,
      0, FLB_TRUE, offsetof(struct flb_net_setup, source_address),
@@ -123,6 +136,12 @@ struct mk_list *flb_upstream_get_config_map(struct flb_config *config)
                 upstream_net[config_index].def_value = config->dns_resolver;
             }
         }
+        if (config->dns_prefer_ipv4) {
+            if (strcmp(upstream_net[config_index].name,
+                       "net.dns.prefer_ipv4") == 0) {
+                upstream_net[config_index].def_value = "true";
+            }
+        }
     }
 
     config_map = flb_config_map_create(config, upstream_net);
@@ -806,23 +825,26 @@ int flb_upstream_conn_timeouts(struct mk_list *list)
                 drop = FLB_TRUE;
 
                 if (!flb_upstream_is_shutting_down(u)) {
-                    flb_error("[upstream] connection #%i to %s:%i timed out after "
-                              "%i seconds",
-                              u_conn->fd,
-                              u->tcp_host, u->tcp_port, u->net.connect_timeout);
+
+                    if (u->net.connect_timeout_log_error) {
+                        flb_error("[upstream] connection #%i to %s:%i timed out after "
+                                  "%i seconds",
+                                  u_conn->fd,
+                                  u->tcp_host, u->tcp_port, u->net.connect_timeout);
+                    }
+                    else {
+                        flb_debug("[upstream] connection #%i to %s:%i timed out after "
+                                  "%i seconds",
+                                  u_conn->fd,
+                                  u->tcp_host, u->tcp_port, u->net.connect_timeout);
+                    }
                 }
             }
 
             if (drop == FLB_TRUE) {
-                /*
-                 * Shutdown the connection, this is the safest way to indicate
-                 * that the socket cannot longer work and any co-routine on
-                 * waiting for I/O will receive the notification and trigger
-                 * the error to it caller.
-                 */
-                if (u_conn->fd != -1) {
-                    shutdown(u_conn->fd, SHUT_RDWR);
-                }
+                mk_event_inject(u_conn->evl, &u_conn->event,
+                                MK_EVENT_READ | MK_EVENT_WRITE,
+                                FLB_TRUE);
 
                 u_conn->net_error = ETIMEDOUT;
                 prepare_destroy_conn(u_conn);
