@@ -9,7 +9,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size){
 
     /* Target the conversion of raw msgpack to gelf */
     flb_sds_t record;
-    struct flb_time tm;
+    struct flb_time tm = {0};
     struct flb_gelf_fields fields = {0};
     fields.short_message_key = flb_sds_create("AAAAAAAAAA");
     record = flb_msgpack_raw_to_gelf(data, size, &tm, &fields);
